#####
#
# Convert SIMO simulated data to CSV files
# 2020-02-21
#
#####

rm(list = ls())  # remove all files in memory


library(dplyr)


### Set the working path to this R-project
path <- paste0(getwd())


### Define the input & output folders
# ------------------------------------

#   Input: contains all .db and will convert them to 
#   to .csv in 'output' folder
inputFolder  <- paste(path, "input_data", sep = "/")
outputFolder <- paste(path, "output", sep = "/")



### Simulation variant that will be load
### The corresponding db files need to be stored in folder "input_data"
### The output .csv files will be stored in folder "output" 
### specify which one should be read:


# FOR MSC STUDENTS - total watershed have been already simulated:
# variant simulated with WIN version of NON-climate sensitive SIMO version
#                    "without"     no climate change
#                    "without_SA"  no climate change and set aside without deadwood extraction (no other management regimes!)
# 
# variants simulated with WIN version of climate sensitive SIMO version, climate model HadGEM2
#                    "CC45"        climate change with RCP scenraio 4.5 
#                    "CC45_SA"     RCP 4.5 and set aside without deadwood extraction (no other management regimes!)
#                    "CC85"        climate change with RCP scenario 8.5 
#                    "CC85_SA"     RCP 8.5 and set aside without deadwood extraction (no other management regimes!)


# FOR COMPARISON OF CLIMATE MODEL AND SIMO VERSIONS - ONLY 5 stands per watershed so far simulated
# variants simulated with SIMO Linux version on cPouta, indicated by "p"
# with climate model HadGEM2
#                   "CC45_p"      RCP scenario 4.5
#                   "CC85_p"      RCP scenario 8.5
#                   "CC26_p"      NEW: RCP scenario 2.6
#                   "CC0_p        NEW: RCP scenario 0, represents NO CLIMATE CHANGE ?? 
#
# with climate model CanESM2 (like in FutureBioEcon), indicated further by "canesm"
#                   "CC45_p_canesm"      RCP sceanrio 4.5
#                   "CC85_p_canesm"      RCP scenario 8.5
#                   "CC26_p_canesm"      NEW: RCP scenario 2.6
#                   "CC0_p_canesm"       NEW: RCP scenario 0


# Select the variation 
# !!! no needed if query based on the file name
#sim_variant <- "CC0_p_canesm"
#sim_variant <- "without"

# Vector to keep all variants for future use:
#all.variants <- c("CC45", 
                  # "CC85", 
                  # "without", 
                  # "CC45_p", 
                  # "CC85_p", 
                  # "CC26_p", 
                  # "CC0_p", 
                  # "CC45_p_canesm", 
                  # "CC85_p_canesm", 
                  # "CC26_p_canesm", 
                  # "CC0_p_canesm")



# List all .db in 'input'
db_input <- list.files(inputFolder,
                       pattern = ".db$")


#Remove the ending ".db" from the list to create a database name
db_names <- gsub(".db", "", db_input)



# Identify columns from UNIT table to export to .cvs
# -----------------------------------------
# different columns are stored in stratum or in comp_unit
# if more variables are required, need to update the SQL 
# created script 'structure_SIMO_rslDB.R'
# An overview on all available SIMO outcomes can be found under: params/Overview_outcomes_SIMO.xlsx

columns <-  paste0("id,
                   year,
                   branch,
                   branch_desc,
                   branching_group,
                   Age,
                   area,
                   cash_flow,
                   V_total_deadwood,
                   BA,
                   V,
                   SC,
                   N,
                   H_dom,
                   D_gm,
                   Harvested_V,
                   Biomass,
                   income_biomass,
                   CARBON_STORAGE, 
                   MAIN_SP")



# If the _SA is included in the .db name
# use specific query


# CHeck for matching character string 
# !!!!!! this needs to be fixed! 
#  what is the correct order of teh queries?
# need to include OPER2 & OPER3 in SA scenario???
# error: op_res.income - in SA and NO SA scenarios

# TRy to add THIN value to SA_scenario???
# NEED to fix final export of csv tables: simplify the output name!!! 
# remove the 'simulated_'

# 
for (name in db_names) {
  print("Back in main!!")
  # CHeck if the name contains "_SA" regime
  # if YES - run SA query
  # if NO  - run normal query
  
  
  if(grepl("_SA", name)){
    
     print(name)
     print("run SA query") 
     # This needs specific SQL, since the database does not contain harvest information
     source(paste0(path, "/structure_SIMO_rslDB_SA.R"))
     
   } else {
     print(name)
     print("run FBE query")
     
     # Run the script with the SQL query for all management regimes besides SA
     source(paste0(path, "/structure_SIMO_rslDB_FBE.R")) # Query 
     
   }
 }
 
 


###
# Create the final CSV for each database
# ----------------------------------

source(paste(path, "loadDB.R", sep = "/")) 




# If one of the follwing simulation variants is read ...
# if(sim_variant %in% all.variants ) {
#   print("sim_varian is in all variants")
#     
#   # Run the script with the SQL query for all management regimes
#   source(paste0(path, "/structure_SIMO_rslDB_FBE.R"))  
#   
#   
#   } else {

#   # Otherwise, run the script witht the SQL query for Set aside without deadwood extraction (...SA)
#   # This needs a different SQL query, since the database does not contain harvest information
#   source(paste0(path, "/structure_SIMO_rslDB_SA.R")) # Query 
# }








### !!! CSV files are stored unter output
#
# If data/columns have already been loaded and saved as csv file  under ../output:                csv_exist = TRUE
# If data is loaded for the firest time OR "new columns have to be loaded" (takes some time):     csv_exits = FALSE 


#csv_exist = TRUE
#
#
### the following lines do not need any changes ###
#if(csv_exist == FALSE){
 # 
  # If FALSE run the following script 
  #source(paste0(path, "loadDB.R")) 
 #
  #} else {
  #
   # # If TRUE read the CSV file that contains all watersheds (GPKGs)
    #rslt <- read.csv(paste0(path, "output/rslt_", sim_variant, "_all.csv" ), sep = ";", header = TRUE, stringsAsFactors = FALSE)
#
 #   }
###






